import { ActionFormData, ModalFormData, MessageFormData } from '@minecraft/server-ui';
import { world, system } from '@minecraft/server';

const DEATH_DATA_TAG = "deathLocations";


world.afterEvents.entityDie.subscribe((event) => {
    const player = event.deadEntity;
    if (player.typeId === "minecraft:player") {
        let deathLocations = getDeathLocations(player);
        deathLocations.push({
            x: player.location.x,
            y: player.location.y,
            z: player.location.z,
            dimensionId: player.dimension.id
        });
        saveDeathLocations(player, deathLocations);
    }
});


world.afterEvents.itemUse.subscribe((event) => {
    const player = event.source;
    const item = event.itemStack;

    if (item?.typeId === "minecraft:recovery_compass") {
        const deathLocations = getDeathLocations(player);

        if (deathLocations.length === 0) {
            player.sendMessage("§eNo death locations recorded.");
            return;
        }

        const form = new ActionFormData()
            .title("Death Locations")
            .body("Select a location to teleport:");

        deathLocations.forEach((loc, index) => {
            form.button(`Death ${index + 1} (${getDimensionName(loc.dimensionId)})`);
        });

        form.button("§cDelete All");

        system.runTimeout(() => {
            form.show(player).then((response) => {
                if (response.canceled) return;

                const selectedIndex = response.selection;
                if (selectedIndex < deathLocations.length) {
                    const selectedLocation = deathLocations[selectedIndex];
                    showTeleportMenu(player, selectedLocation, selectedIndex);
                } else {
                    confirmDeleteAll(player);
                }
            });
        }, 2);
    }
});


function showTeleportMenu(player, location, index) {
    const teleportForm = new MessageFormData()
        .title(`Death Location #${index + 1}`)
        .body(`Coordinates:\nX: ${location.x.toFixed(1)}\nY: ${location.y.toFixed(1)}\nZ: ${location.z.toFixed(1)}\nDimension: ${getDimensionName(location.dimensionId)}\n\nUse 1 Ender Pearl to teleport.`)
        .button1("Teleport")
        .button2("Show Coordinates");

    system.runTimeout(() => {
        teleportForm.show(player).then((response) => {
            if (response.canceled) return;

            if (response.selection === 1) {
                player.sendMessage(`§bDeath Location:\nX: ${location.x.toFixed(1)}, Y: ${location.y.toFixed(1)}, Z: ${location.z.toFixed(1)}\nDimension: ${getDimensionName(location.dimensionId)}`);
            } else if (response.selection === 0) {
                attemptTeleport(player, location);
            }
        });
    }, 2);
}

function attemptTeleport(player, location) {
    const inventory = player.getComponent("minecraft:inventory")?.container;
    if (!inventory) return;

    let pearlSlot = -1;
    for (let i = 0; i < inventory.size; i++) {
        const item = inventory.getItem(i);
        if (item?.typeId === "minecraft:ender_pearl") {
            pearlSlot = i;
            break;
        }
    }

    if (pearlSlot === -1) {
        player.sendMessage("§cYou need an Ender Pearl to teleport!");
        return;
    }

    try {
        const pearlItem = inventory.getItem(pearlSlot);
        if (pearlItem.amount > 1) {
            pearlItem.amount -= 1;
            inventory.setItem(pearlSlot, pearlItem);
        } else {
            inventory.setItem(pearlSlot, undefined);
        }

        const targetDimension = world.getDimension(location.dimensionId);
        if (!targetDimension) throw new Error("Invalid dimension");

        player.teleport(
            { x: location.x, y: location.y, z: location.z },
            { dimension: targetDimension, checkForBlocks: false }
        );

        player.sendMessage("§aTeleported to your death location!");
    } catch (e) {
        player.sendMessage(`§cTeleport failed: ${e.message}`);
    }
}


function confirmDeleteAll(player) {
    const confirmForm = new MessageFormData()
        .title("Confirm Delete")
        .body("Are you sure you want to delete all saved death locations?")
        .button1("§cConfirm")
        .button2("Cancel");

    system.runTimeout(() => {
        confirmForm.show(player).then((response) => {
            if (response.selection === 0) {
                saveDeathLocations(player, []);
                player.sendMessage("§aAll death locations have been deleted.");
            }
        });
    }, 2);
}


function saveDeathLocations(player, locations) {
    player.setDynamicProperty(DEATH_DATA_TAG, JSON.stringify(locations));
}


function getDeathLocations(player) {
    const data = player.getDynamicProperty(DEATH_DATA_TAG);
    return data ? JSON.parse(data) : [];
}

function getDimensionName(id) {
    switch (id) {
        case "minecraft:overworld": return "Overworld";
        case "minecraft:nether": return "Nether";
        case "minecraft:the_end": return "The End";
        default: return "Unknown";
    }
}


system.runInterval(() => {
    world.getDimension("overworld").runCommandAsync("clear @a[hasitem={item=recovery_compass, quantity=2..}] recovery_compass 0 1");
    world.getDimension("overworld").runCommandAsync("clear @a[hasitem={item=recovery_compass, quantity=31..}] recovery_compass 0 30");
    world.getDimension("overworld").runCommandAsync("clear @a[hasitem={item=recovery_compass, quantity=65..}] recovery_compass 0 64");
    world.getDimension("overworld").runCommandAsync("clear @a[hasitem={item=recovery_compass, quantity=129..}] recovery_compass 0 128");
}, 20);