import { ActionFormData, ModalFormData } from '@minecraft/server-ui';
import { world, system } from '@minecraft/server';

const locationTypes = [
    "Home", "Spawn", "Secret Base", "Village", "Boss",
    "Dungeon", "Industrial Area", "Shop", "Nether Portal",
    "End Portal", "Strong Hold", "Forest", "Other"
];

function loadPlayerLocations(player) {
    const savedLocations = player.getDynamicProperty('savedLocations');
    return savedLocations ? JSON.parse(savedLocations) : [];
}

function savePlayerLocations(player, locations) {
    player.setDynamicProperty('savedLocations', JSON.stringify(locations));
}

function summonFireworksAtLocation(player, location) {
    if (!location || !location.position) return;
    const { x, y, z } = location.position;
    player.dimension.spawnEntity("minecraft:fireworks_rocket", { x, y: y + 1, z });
    player.sendMessage(`§6Fireworks launched at ${location.name}!`);
}

function removeEnderPearl(player) {
    const inventory = player.getComponent("minecraft:inventory").container;
    for (let i = 0; i < inventory.size; i++) {
        const item = inventory.getItem(i);
        if (item && item.typeId === "minecraft:ender_pearl" && item.amount > 0) {
            item.amount -= 1;
            if (item.amount === 0) {
                inventory.setItem(i, undefined);
            } else {
                inventory.setItem(i, item);
            }
            return true;
        }
    }
    return false;
}

function teleportPlayer(player, location) {
    if (removeEnderPearl(player)) {
        system.run(async () => {
            await player.runCommandAsync("effect @s blindness 3 3 true");
            player.teleport(location.position, { dimension: player.dimension });
            player.sendMessage(`§aTeleported to §6${location.name}!`);
            summonFireworksAtLocation(player, location);
        });
    } else {
        player.sendMessage("§cYou need 1 Ender Pearl to teleport!");
    }
}

function showTeleportOptions(player, location) {
    const teleportUI = new ActionFormData()
        .title(`§l§5${location.name}`)
        .body(`Coordinates:\n§aX: ${location.position.x.toFixed(2)}, Y: ${location.position.y.toFixed(2)}, Z: ${location.position.z.toFixed(2)}`)
        .button("Teleport (Cost: 1 Ender Pearl)", "textures/items/ender_pearl")
        .button("No, Just Show Coordinates", "textures/ui/magnifyingGlass");

    teleportUI.show(player).then((response) => {
        if (!response.canceled) {
            if (response.selection === 0) {
                teleportPlayer(player, location);
            } else {
                player.sendMessage(`§bLocation Details:\n§aX: ${location.position.x.toFixed(2)}, Y: ${location.position.y.toFixed(2)}, Z: ${location.position.z.toFixed(2)}`);
            }
        }
    });
}

function showCombinedUI(player, locations) {
    const combinedUI = new ActionFormData()
        .title("§l§2Manage Locations")
        .body("Submit A New Location:")
        .button("Submit a New Location", "textures/ui/editIcon")
        .button("Delete Marked Place", "textures/ui/trash")
        .button("Search Location", "textures/ui/magnifyingGlass");

    locations.forEach(location => {
        const locationName = location.name || "Unnamed Location";
        const locationType = locationTypes[location.type] || "Unknown";
        const publicStatus = location.public ? "\n§a[Public]" : "";
        combinedUI.button(`§l§5${locationName}\n§4Type: ${locationType}${publicStatus}`, "textures/ui/mashup_world");
    });

    combinedUI.show(player).then((response) => {
        if (!response.canceled) {
            if (response.selection === 0) {
                showAddLocationForm(player, locations);
            } else if (response.selection === 1) {
                showDeleteLocationForm(player, locations);
            } else if (response.selection === 2) {
                showSearchLocationForm(player, locations);
            } else {
                const selectedLocation = locations[response.selection - 3];
                if (selectedLocation) {
                    showTeleportOptions(player, selectedLocation);
                }
            }
        }
    });
}

function showAddLocationForm(player, locations) {
    const addLocationForm = new ModalFormData()
        .title("§l§2Add New Location")
        .dropdown("Choose a location type:", locationTypes, 0)
        .textField("Enter a name for your location:", "Type here...")
        .toggle("Make this location public?", false);  // Public toggle added

    addLocationForm.show(player).then((response) => {
        if (!response.canceled) {
            const locationTypeIndex = response.formValues[0];
            const customName = response.formValues[1].trim();
            const isPublic = response.formValues[2];  // Get public value
            const locationName = customName.length > 0 ? customName : "No Name";
            const playerPosition = player.location;

            locations.push({
                name: locationName,
                type: locationTypeIndex,
                position: playerPosition,
                public: isPublic
            });

            savePlayerLocations(player, locations);
            showCombinedUI(player, locations);
        }
    });
}

function showDeleteLocationForm(player, locations) {
    const deleteForm = new ModalFormData()
        .title("§l§4Delete Marked Place")
        .dropdown("Select a location to delete:", locations.map(loc => loc.name), 0);

    deleteForm.show(player).then((response) => {
        if (!response.canceled) {
            const selectedIndex = response.formValues[0];
            locations.splice(selectedIndex, 1);
            savePlayerLocations(player, locations);
            showCombinedUI(player, locations);
        }
    });
}

function showSearchLocationForm(player, locations) {
    const searchForm = new ModalFormData()
        .title("§l§9Search Location")
        .textField("Enter location name to search:", "Type here...");

    searchForm.show(player).then((response) => {
        if (!response.canceled) {
            const query = response.formValues[0].toLowerCase().trim();
            const results = locations.filter(loc => loc.name.toLowerCase().includes(query));
            if (results.length > 0) {
                showSearchResultsForm(player, results);
            } else {
                player.sendMessage("§cNo locations found with that name.");
            }
        }
    });
}

function showSearchResultsForm(player, results) {
    const resultUI = new ActionFormData()
        .title("§l§3Search Results")
        .body("Select a location:");

    results.forEach(loc => {
        const locType = locationTypes[loc.type] || "Unknown";
        const publicStatus = loc.public ? "\n§a[Public]" : "";
        resultUI.button(`§l§5${loc.name}\n§4Type: ${locType}${publicStatus}`, "textures/ui/mashup_world");
    });

    resultUI.show(player).then((response) => {
        if (!response.canceled) {
            const selectedLocation = results[response.selection];
            if (selectedLocation) {
                showTeleportOptions(player, selectedLocation);
            }
        }
    });
}

world.afterEvents.itemUse.subscribe((event) => {
    const { source, itemStack } = event;
    if (itemStack.typeId === "zc:location_diary") {
        const locations = loadPlayerLocations(source);
        showCombinedUI(source, locations);
    }
});
