import { ActionFormData, ModalFormData } from '@minecraft/server-ui';
import { world } from '@minecraft/server';

function loadPlayerLocations(player) {
    const savedLocations = player.getDynamicProperty('savedLocations');
    if (savedLocations) {
        return JSON.parse(savedLocations);
    }
    return [];
}

function savePlayerLocations(player, locations) {
    player.setDynamicProperty('savedLocations', JSON.stringify(locations));
}

function showCombinedUI(player, locations) {
    const combinedUI = new ActionFormData()
        .title("§l§dManage Locations")
        .body("Select a location to teleport or manage:")
        .button("Delete Marked Place", "textures/ui/trash");

    locations.forEach(location => {
        combinedUI.button(
            `§l§5${location.name}§r\n§8[${location.type}] ${location.public ? '§1(Public)' : '§4(Private)'}`, "textures/ui/mashup_world");
    });

    combinedUI.show(player).then((response) => {
        if (!response.canceled) {
            if (response.selection === 0) {
                showDeleteLocationForm(player, locations);
            } else {
                const selectedLocation = locations[response.selection - 1];
                if (player.level >= 2) {
                    player.teleport(selectedLocation.position, { dimension: world.getDimension(selectedLocation.dimension) });
                    player.addLevels(-2);
                    player.sendMessage("§aTeleported successfully! 2 XP levels were deducted.");
                } else {
                    player.sendMessage("§cYou need at least 2 XP levels to teleport to this location.");
                }
            }
        }
    });
}

function showAddLocationForm(player, locations) {
    const locationTypes = [
        "Home", "Spawn", "Secret Base", "Village", "Boss", "Dungeon",
        "Industrial Area", "Shop", "Nether Portal", "End Portal",
        "Strong Hold", "Forest", "Other"
    ];

    const addLocationForm = new ModalFormData()
        .title("§l§2Add New Location")
        .dropdown("Choose a location type:", locationTypes, 0)
        .textField("Enter a name for your location:", "Type here...")
        .toggle("Make this location public?", false);

    addLocationForm.show(player).then((response) => {
        if (response.canceled) {
            const playerPosition = player.location;
            const dimensionId = player.dimension.id;
            
            locations.push({
                name: "No Name",
                type: "Other",
                position: playerPosition,
                dimension: dimensionId,
                public: false
            });

            savePlayerLocations(player, locations);
            player.sendMessage("§aLocation saved with default values (No Name, Other, Private)");
        } else {
            const locationType = locationTypes[response.formValues[0]];
            const customName = response.formValues[1].trim();
            const isPublic = response.formValues[2];
            const locationName = customName.length > 0 ? customName : "No Name";
            const playerPosition = player.location;
            const dimensionId = player.dimension.id;

            locations.push({
                name: locationName,
                type: locationType,
                position: playerPosition,
                dimension: dimensionId,
                public: isPublic
            });

            savePlayerLocations(player, locations);
            player.sendMessage(`§aLocation '${locationName}' saved!`);
        }
    });
}

function showDeleteLocationForm(player, locations) {
    const deleteForm = new ModalFormData()
        .title("§l§4Delete Marked Place")
        .dropdown("Select a location to delete:", locations.map(loc => `${loc.name} (${loc.type})`), 0);

    deleteForm.show(player).then((response) => {
        if (!response.canceled) {
            const selectedIndex = response.formValues[0];
            locations.splice(selectedIndex, 1);
            savePlayerLocations(player, locations);
            player.sendMessage("§cLocation deleted.");
        }
    });
}


world.afterEvents.playerPlaceBlock.subscribe((event) => {
    const player = event.player;
    const block = event.block;

    if (block.typeId === "zc:teleport_stone1") {
        const locations = loadPlayerLocations(player);
        showAddLocationForm(player, locations);
    }
});


world.beforeEvents.worldInitialize.subscribe((initEvent) => {
    initEvent.blockComponentRegistry.registerCustomComponent('zc_tele:trigger', {
        onPlayerInteract: (e) => {
            const player = e.player;
            if (player.level > 1) {
                const locations = loadPlayerLocations(player);
                showCombinedUI(player, locations);
            } else {
                player.sendMessage("§cYou need at least 2 XP levels to use Teleport Stone.");
            }
        },
    });
});

function getPublicLocations() {
    let publicLocations = [];
    for (const player of world.getAllPlayers()) {
        let locations = loadPlayerLocations(player);
        publicLocations = publicLocations.concat(locations.filter(loc => loc.public));
    }
    return publicLocations;
}